/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.se.flowable.clientserver.api.admin.AdminRuntimeService;

/**
 * The command 'se-flowable.list-process-instances'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ListProcessInstances extends AbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "list-process-instances";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "List process instances of the SE Flowable";

    /**
     * The option associated to the process definition key
     */
    public static final String PROC_DEF_KEY_SHORT_OPTION = "k";

    private static final String PROC_DEF_KEY_LONG_OPTION = "process-definition-key";

    private static final String PROC_DEF_KEY_ARG_NAME = "process-definition-key";

    private static final Option PROC_DEF_KEY_OPTION = Option.builder(PROC_DEF_KEY_SHORT_OPTION)
            .longOpt(PROC_DEF_KEY_LONG_OPTION).required(false).hasArg().numberOfArgs(1).argName(PROC_DEF_KEY_ARG_NAME)
            .desc("The process definition key of process instances to retrieve.").build();

    /**
     * The option associated to the process definition version
     */
    public static final String PROC_DEF_VER_SHORT_OPTION = "v";

    private static final String PROC_DEF_VER_LONG_OPTION = "process-definition-version";

    private static final String PROC_DEF_VER_ARG_NAME = "process-definition-version";

    private static final Option PROC_DEF_VER_OPTION = Option.builder(PROC_DEF_VER_SHORT_OPTION)
            .longOpt(PROC_DEF_VER_LONG_OPTION).required(false).hasArg().numberOfArgs(1).argName(PROC_DEF_VER_ARG_NAME)
            .desc("The process definition version of process instances to retrieve.").build();

    /**
     * The option associated to the output file name
     */
    public static final String OUTPUTFILE_SHORT_OPTION = "o";

    private static final String OUTPUTFILE_LONG_OPTION = "output-file";

    private static final String OUTPUTFILE_ARG_NAME = "output-file";

    private static final Option OUTPUTFILE_OPTION = Option.builder(OUTPUTFILE_SHORT_OPTION)
            .longOpt(OUTPUTFILE_LONG_OPTION).required(false).hasArg().numberOfArgs(1).argName(OUTPUTFILE_ARG_NAME)
            .desc("Name of the output file where process instance identifiers are printed.").build();

    public ListProcessInstances() {
        super(NAME);
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription(DESCRIPTION);
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(PROC_DEF_KEY_OPTION).addOption(PROC_DEF_VER_OPTION)
                .addOption(OUTPUTFILE_OPTION);
    }

    @Override
    public final void doExecute(final String[] args) throws CommandException {
        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // The method 'parse(...)' throws an UnrecognizedOptionException if there is an options starting with "-"
            // that is unknown, but arguments that are not understood are simply added to the args of CLI. In our case,
            // we don't want any or there is an error somewhere.
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            final String componentId;
            if (cli.hasOption(COMPONENT_ID_SHORT_OPTION)) {
                componentId = cli.getOptionValue(COMPONENT_ID_SHORT_OPTION);
            } else {
                componentId = DEFAULT_COMPONENT_ID;
            }

            final String outputFileName;
            if (cli.hasOption(OUTPUTFILE_SHORT_OPTION)) {
                outputFileName = cli.getOptionValue(OUTPUTFILE_SHORT_OPTION);
            } else {
                outputFileName = null;
            }

            final List<String> procInsts;
            if (cli.hasOption(PROC_DEF_KEY_SHORT_OPTION) && cli.hasOption(PROC_DEF_VER_SHORT_OPTION)) {
                procInsts = this.listProcessInstances(cli.getOptionValue(PROC_DEF_KEY_SHORT_OPTION),
                        cli.getOptionValue(PROC_DEF_VER_SHORT_OPTION), componentId);
            } else {
                procInsts = this.listProcessInstances(null, null, componentId);
            }

            this.printProcessInstances(procInsts, outputFileName);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        }
    }

    /**
     * Retrieve process instances to list
     * 
     * @param procDefKey
     *            process definition key. Can be {@code null} and empty.
     * @param procDefVersion
     *            process definition version. Can be {@code null} and empty.
     * @param componentId
     *            Component JBI identifier. Not {@code null} and not empty.
     * @return List of process instances
     */
    private List<String> listProcessInstances(final String procDefKey, final String procDefVersion,
            final String componentId) throws CommandException {
        assert componentId != null;
        assert !componentId.trim().isEmpty();

        try {
            final JMXClient client = this.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);

            if (procDefVersion != null && !procDefVersion.trim().isEmpty()) {
                return admin.listPurgeableProcessInstances(procDefKey, Integer.parseInt(procDefVersion));
            } else {
                return admin.listPurgeableProcessInstances(procDefKey, 0);
            }

        } catch (final NumberFormatException e) {
            throw new CommandUnparsableArgumentException(this, PROC_DEF_VER_OPTION, procDefVersion,
                    "Integer value expected");
        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }

    private void printProcessInstances(final List<String> procInsts, final String outputFilename)
            throws CommandException {

        try {
            if (outputFilename == null) {
                this.getShell().getPrintStream().println("Process instance identifiers matching criteria are:");
                printProcessInstanceIds(procInsts, this.getShell().getPrintStream());
            } else {
                try (final FileOutputStream fos = new FileOutputStream(outputFilename);
                        final PrintStream procInstIdsOut = new PrintStream(fos);) {
                    this.getShell().getPrintStream()
                            .format("Flushing process instance identifiers matching criteria in file %s ...%n",
                                    outputFilename);
                    printProcessInstanceIds(procInsts, procInstIdsOut);
                }
            }
            this.getShell().getPrintStream().format("%d process instance(s) returned.", procInsts.size());
        } catch (final FileNotFoundException e) {
            throw new CommandInvalidArgumentException(this, OUTPUTFILE_OPTION, outputFilename, e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        }

    }

    private void printProcessInstanceIds(final List<String> procInsts, final PrintStream procInstIdsOut) {
        for (final String procInst : procInsts) {
            procInstIdsOut.println(procInst);
        }
    }
}
