/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.apache.commons.io.FileUtils;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.se.flowable.clientserver.api.admin.AdminRuntimeService;

/**
 * The command 'se-flowable.purge-process-instances'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PurgeProcessInstance extends AbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "purge-process-instance";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "Purge a process instance of the SE Flowable";

    /**
     * The option associated to the process instance
     */
    public static final String PROC_INST_SHORT_OPTION = "i";

    private static final String PROC_INST_LONG_OPTION = "process-instance";

    private static final String PROC_INST_ARG_NAME = "process-instance";

    private static final Option PROC_INST_OPTION = Option.builder(PROC_INST_SHORT_OPTION).longOpt(PROC_INST_LONG_OPTION)
            .required(true).numberOfArgs(1).argName(PROC_INST_ARG_NAME).desc("The process instance to purge.").build();

    /**
     * The option associated to the input file name
     */
    public static final String INPUTFILE_SHORT_OPTION = "f";

    private static final String INPUTFILE_LONG_OPTION = "process-instance-ids-file";

    private static final String INPUTFILE_ARG_NAME = "process-instance-ids-file";

    private static final Option INPUTFILE_OPTION = Option.builder(INPUTFILE_SHORT_OPTION).longOpt(INPUTFILE_LONG_OPTION)
            .required(false).hasArg().numberOfArgs(1).argName(INPUTFILE_ARG_NAME)
            .desc("Name of the input file containing identifiers of process instances to purge.").build();

    /**
     * The option associated to the output file name
     */
    public static final String OUTPUTFILE_SHORT_OPTION = "o";

    private static final String OUTPUTFILE_LONG_OPTION = "output-file";

    private static final String OUTPUTFILE_ARG_NAME = "output-file";

    private static final Option OUTPUTFILE_OPTION = Option.builder(OUTPUTFILE_SHORT_OPTION)
            .longOpt(OUTPUTFILE_LONG_OPTION).required(false).hasArg().numberOfArgs(1).argName(OUTPUTFILE_ARG_NAME)
            .desc("Name of the output file where process instance identifiers are printed.").build();

    public PurgeProcessInstance() {
        super(NAME);
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription(DESCRIPTION);
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public Options createOptions() {
        final OptionGroup optGrp = new OptionGroup();
        optGrp.addOption(PROC_INST_OPTION).addOption(INPUTFILE_OPTION);
        return super.createOptions().addOptionGroup(optGrp).addOption(OUTPUTFILE_OPTION);
    }

    @Override
    public final void doExecute(final String[] args) throws CommandException {
        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // The method 'parse(...)' throws an UnrecognizedOptionException if there is an options starting with "-"
            // that is unknown, but arguments that are not understood are simply added to the args of CLI. In our case,
            // we don't want any or there is an error somewhere.
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            final String componentId;
            if (cli.hasOption(COMPONENT_ID_SHORT_OPTION)) {
                componentId = cli.getOptionValue(COMPONENT_ID_SHORT_OPTION);
            } else {
                componentId = DEFAULT_COMPONENT_ID;
            }

            final String outputFileName;
            if (cli.hasOption(OUTPUTFILE_SHORT_OPTION)) {
                outputFileName = cli.getOptionValue(OUTPUTFILE_SHORT_OPTION);
            } else {
                outputFileName = null;
            }

            final List<String> flowInstIds;
            if (cli.hasOption(PROC_INST_SHORT_OPTION)) {
                flowInstIds = this.purgeProcessInstance(cli.getOptionValue(PROC_INST_SHORT_OPTION), componentId);
            } else if (cli.hasOption(INPUTFILE_SHORT_OPTION)) {
                final List<String> procInstId = this.readProcInstIds(cli.getOptionValue(INPUTFILE_SHORT_OPTION));
                flowInstIds = this.purgeProcessInstances(procInstId, componentId);
            } else {
                // Should not occurs because the exception MissingOptionException is thrown on arguments parsing
                throw new CommandMissingOptionsException(this, Arrays.asList(PROC_INST_SHORT_OPTION));
            }

            this.printFlowInstanceIds(flowInstIds, outputFileName);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        }
    }

    /**
     * Purge a process instance
     * 
     * @param procInstId
     *            process instance identifier. Not {@code null} and not empty.
     * @param componentId
     *            Component JBI identifier. Not {@code null} and not empty.
     * @return List of process instances
     */
    private List<String> purgeProcessInstance(final String procInstId,
            final String componentId) throws CommandException {
        assert procInstId != null;
        assert !procInstId.trim().isEmpty();
        assert componentId != null;
        assert !componentId.trim().isEmpty();

        try {
            final JMXClient client = this.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);

            return admin.purgeProcessInstance(procInstId, true);

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     * Read process instance identifiers from a file
     * 
     * @param inputFileName
     *            The file name
     * @return Process instance identifiers read from the given file
     * @throws CommandException
     *             An error occurs reading file
     */
    private List<String> readProcInstIds(final String inputFileName) throws CommandException {
        try {
            final List<String> procInstIds = new LinkedList<>();
            final List<String> lines = FileUtils.readLines(new File(inputFileName), Charset.defaultCharset());
            for (final String line : lines) {
                final String trimmedLine = line.trim();
                if (!trimmedLine.isEmpty() && !trimmedLine.startsWith("#") && !trimmedLine.startsWith("//")) {
                    procInstIds.add(line);
                }
            }
            return procInstIds;
        } catch (final FileNotFoundException e) {
            throw new CommandInvalidArgumentException(this, INPUTFILE_OPTION, inputFileName, e);
        } catch (final IOException e) {
            if (e.getMessage().contains("is a directory") || e.getMessage().contains("cannot be read")) {
                throw new CommandInvalidArgumentException(this, INPUTFILE_OPTION, inputFileName, e);
            } else {
                throw new CommandException(this, e);
            }
        }
    }

    /**
     * Purge a process instance list
     * 
     * @param procInstIds
     *            process instance identifiers. Not {@code null}.
     * @param componentId
     *            Component JBI identifier. Not {@code null} and not empty.
     * @return List of process instances
     */
    private List<String> purgeProcessInstances(final List<String> procInstIds, final String componentId)
            throws CommandException {
        assert procInstIds != null;
        assert componentId != null;
        assert !componentId.trim().isEmpty();

        try {
            final JMXClient client = this.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);

            final List<String> result = new LinkedList<>();
            for (final String procInstId : procInstIds) {
                result.addAll(admin.purgeProcessInstance(procInstId, true));
            }
            return result;

        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }

    private void printFlowInstanceIds(final List<String> flowInsts, final String outputFilename)
            throws CommandException {

        try {
            if (outputFilename == null) {
                this.getShell().getPrintStream().println("Associated flow instance identifiers are:");
                printFlowInstanceIds(flowInsts, this.getShell().getPrintStream());
            } else {
                try (final FileOutputStream fos = new FileOutputStream(outputFilename);
                        final PrintStream flowInstIdsOut = new PrintStream(fos);) {
                    this.getShell().getPrintStream()
                            .format("Flushing associated flow instance identifiers in file %s ...%n", outputFilename);
                    printFlowInstanceIds(flowInsts, flowInstIdsOut);
                }
            }
            this.getShell().getPrintStream().format("%d associated flow instance(s) returned.", flowInsts.size());
        } catch (final FileNotFoundException e) {
            throw new CommandInvalidArgumentException(this, OUTPUTFILE_OPTION, outputFilename, e);
        } catch (final IOException e) {
            throw new CommandException(this, e);
        }

    }

    private void printFlowInstanceIds(final List<String> procInsts, final PrintStream procInstIdsOut) {
        for (final String procInst : procInsts) {
            procInstIdsOut.println(procInst);
        }
    }
}
