/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.se.flowable.clientserver.api.admin.AdminRuntimeService;

/**
 * The command 'se-flowable.undeploy-process-definition'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class UndeployProcessDefinition extends AbstractCommand {

    /**
     * The command name
     */
    private static final String NAME = "undeploy-process-definition";

    /**
     * The command description
     */
    private static final String DESCRIPTION = "Undeploy a process definition";

    /**
     * The option associated to the process definition key
     */
    public static final String PROC_DEF_KEY_SHORT_OPTION = "k";

    private static final String PROC_DEF_KEY_LONG_OPTION = "process-definition-key";

    private static final String PROC_DEF_KEY_ARG_NAME = "process-definition-key";

    private static final Option PROC_DEF_KEY_OPTION = Option.builder(PROC_DEF_KEY_SHORT_OPTION)
            .longOpt(PROC_DEF_KEY_LONG_OPTION).required(true).hasArg().numberOfArgs(1).argName(PROC_DEF_KEY_ARG_NAME)
            .desc("The process definition key to undeploy.").build();

    /**
     * The option associated to the process definition version
     */
    public static final String PROC_DEF_VER_SHORT_OPTION = "v";

    private static final String PROC_DEF_VER_LONG_OPTION = "process-definition-version";

    private static final String PROC_DEF_VER_ARG_NAME = "process-definition-version";

    private static final Option PROC_DEF_VER_OPTION = Option.builder(PROC_DEF_VER_SHORT_OPTION)
            .longOpt(PROC_DEF_VER_LONG_OPTION).required(true).hasArg().numberOfArgs(1).argName(PROC_DEF_VER_ARG_NAME)
            .desc("The process definition version to undeploy.").build();

    public UndeployProcessDefinition() {
        super(NAME);
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription(DESCRIPTION);
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public Options createOptions() {
        return super.createOptions().addOption(PROC_DEF_KEY_OPTION).addOption(PROC_DEF_VER_OPTION);
    }

    @Override
    public final void doExecute(final String[] args) throws CommandException {
        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // The method 'parse(...)' throws an UnrecognizedOptionException if there is an options starting with "-"
            // that is unknown, but arguments that are not understood are simply added to the args of CLI. In our case,
            // we don't want any or there is an error somewhere.
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            final String componentId;
            if (cli.hasOption(COMPONENT_ID_SHORT_OPTION)) {
                componentId = cli.getOptionValue(COMPONENT_ID_SHORT_OPTION);
            } else {
                componentId = DEFAULT_COMPONENT_ID;
            }

            this.undeployProcessDefinition(cli.getOptionValue(PROC_DEF_KEY_SHORT_OPTION),
                        cli.getOptionValue(PROC_DEF_VER_SHORT_OPTION), componentId);

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        }
    }

    /**
     * Undeploy a process definition
     * 
     * @param procDefKey
     *            process definition key. Not {@code null} and not empty.
     * @param procDefVersion
     *            process definition version. Not {@code null} and not empty.
     * @param componentId
     *            Component JBI identifier. Not {@code null} and not empty.
     * @return List of process instances
     */
    private void undeployProcessDefinition(final String procDefKey, final String procDefVersion,
            final String componentId) throws CommandException {
        assert procDefKey != null;
        assert !procDefKey.trim().isEmpty();
        assert procDefVersion != null;
        assert !procDefVersion.trim().isEmpty();
        assert componentId != null;
        assert !componentId.trim().isEmpty();

        try {
            final JMXClient client = this.getJMXClient();
            final AdminRuntimeService admin = client.getComponentClient(componentId)
                    .getRuntimeConfigurationClient(AdminRuntimeService.class);

            admin.undeployProcessDefinition(procDefKey, Integer.parseInt(procDefVersion));

        } catch (final NumberFormatException e) {
            throw new CommandUnparsableArgumentException(this, PROC_DEF_VER_OPTION, procDefVersion,
                    "Integer value expected");
        } catch (final PetalsException e) {
            throw new CommandException(this, e);
        }
    }
}
