/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingArgumentException;
import static org.ow2.petals.cli.extension.seflowable.AbstractCommand.COMPONENT_ID_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.OUTPUTFILE_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_KEY_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_VER_SHORT_OPTION;

import java.io.File;
import java.nio.charset.Charset;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.seflowable.mock.PetalsSeFlowableRuntimeConfigurationMock;
import org.ow2.petals.cli.extension.seflowable.mock.ProcDefStruct;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;

/**
 * Unit test of {@link ListProcessInstances}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ListProcessInstancesTest extends AbstractTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void usage() {
        final Command command = new ListProcessInstances();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws Exception {

        final Command command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition key value is missing,</li>
     * <li>an existing process definition version is correctly provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process definition key.
     * </p>
     */
    @Test
    public void missingProcDefKeyValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PROC_DEF_KEY_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" against argument management, where:
     * </p>
     * <ul>
     * <li>an existing process definition key correctly provided,</li>
     * <li>the process definition version value is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process definition version.
     * </p>
     */
    @Test
    public void missingProcDefVerValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PROC_DEF_VER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition version is unparseable</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid value.
     * </p>
     */
    @Test
    public void unparsableProcDefVer() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        final String unparsableValue = "123aze456";
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "procDefKey",
                    "-" + PROC_DEF_VER_SHORT_OPTION, unparsableValue });

            fail("Exception 'CommandUnparsableArgumentException' is not thrown");
        } catch (final CommandUnparsableArgumentException e) {
            assertEquals(PROC_DEF_VER_SHORT_OPTION, e.getOption().getOpt());
            assertEquals(unparsableValue, e.getUnparsableValue());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" against argument management, where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>the component identifier value is missing</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process instance identifier.
     * </p>
     */
    @Test
    public void missingComponentIdValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + AbstractCommand.COMPONENT_ID_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, AbstractCommand.COMPONENT_ID_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: only process instances of the given process definition are printed on the default output.
     * </p>
     */
    @Test
    public void listeProcInsts() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertTrue(output.contains(PROCESS_INSTANCE_11_1));
        assertTrue(output.contains(PROCESS_INSTANCE_11_2));
        assertFalse(output.contains(PROCESS_INSTANCE_12_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_2));
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: only process instances of the given process definition are printed into the output file.
     * </p>
     */
    @Test
    public void listeProcInstsIntoFile() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final File outputFile = this.tempFolder.newFile();

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + OUTPUTFILE_SHORT_OPTION, outputFile.getAbsolutePath() });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        final String outputFileContent = FileUtils.readFileToString(outputFile, Charset.defaultCharset());
        assertTrue(outputFileContent.contains(PROCESS_INSTANCE_11_1));
        assertTrue(outputFileContent.contains(PROCESS_INSTANCE_11_2));
        assertFalse(outputFileContent.contains(PROCESS_INSTANCE_12_1));
        assertFalse(outputFileContent.contains(PROCESS_INSTANCE_21_1));
        assertFalse(outputFileContent.contains(PROCESS_INSTANCE_21_2));
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output file is provided without value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing output file value.
     * </p>
     */
    @Test
    public void missingOutputFile() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + OUTPUTFILE_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, OUTPUTFILE_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output directory is provided for the output file.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the output file that is a directory.
     * </p>
     */
    @Test
    public void outputFileAsDirectory() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final File outputFileAsDir = this.tempFolder.newFolder();
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + OUTPUTFILE_SHORT_OPTION,
                    outputFileAsDir.getAbsolutePath() });

            fail("Exception 'CommandInvalidArgumentException' is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, OUTPUTFILE_SHORT_OPTION, outputFileAsDir.getAbsolutePath());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an unexisting process definition key is correctly provided,</li>
     * <li>an unexisting process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: only process instances of the given process definition are returned.
     * </p>
     */
    @Test
    public void listeUnexistingProcInsts() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "unexisting-process-defintion",
                "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.trim().isEmpty());

        assertFalse(output.contains(PROCESS_INSTANCE_11_1));
        assertFalse(output.contains(PROCESS_INSTANCE_11_2));
        assertFalse(output.contains(PROCESS_INSTANCE_12_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_2));

    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>no process definition key is provided,</li>
     * <li>no process definition version is provided,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: all process instances of the given process definition are returned.
     * </p>
     */
    @Test
    public void listeAllProcInsts() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] {});

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertTrue(output.contains(PROCESS_INSTANCE_11_1));
        assertTrue(output.contains(PROCESS_INSTANCE_11_2));
        assertTrue(output.contains(PROCESS_INSTANCE_12_1));
        assertTrue(output.contains(PROCESS_INSTANCE_21_1));
        assertTrue(output.contains(PROCESS_INSTANCE_21_2));
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>an existing component id is correctly provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: only process instances of the given process definition on the given component are returned.
     * </p>
     */
    @Test
    public void listeProcInstsWithComponentId() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMockDefault = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMockDefault.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMockDefault, null);
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMockOther = new PetalsSeFlowableRuntimeConfigurationMock();
        final String myComponentId = "my-comp-id";
        final String myProcDefKey = "my-proc-def";
        final ProcDefStruct myProcDef = new ProcDefStruct(myProcDefKey, "1");
        final String myProcInst = "my-proc-inst";
        final List<String> myProcInsts = Arrays.asList(myProcInst);
        petalsSeFlowableRuntimeConfigurationMockOther.getProcessInstanceIds().put(myProcDef, myProcInsts);
        this.jmxApi.addComponentClient(myComponentId, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMockOther, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, myProcDefKey, "-" + PROC_DEF_VER_SHORT_OPTION,
                "1", "-" + COMPONENT_ID_SHORT_OPTION, myComponentId });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertFalse(output.contains(PROCESS_INSTANCE_11_1));
        assertFalse(output.contains(PROCESS_INSTANCE_11_2));
        assertFalse(output.contains(PROCESS_INSTANCE_12_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_1));
        assertFalse(output.contains(PROCESS_INSTANCE_21_2));
        assertTrue(output.contains(myProcInst));
    }

    /**
     * <p>
     * Test of the command "se-flowable.list-process-instances" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void notConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new ListProcessInstances();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
