/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;
import static org.ow2.petals.cli.extension.seflowable.AbstractCommand.COMPONENT_ID_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_KEY_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_VER_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.PurgeProcessInstance.PROC_INST_SHORT_OPTION;

import java.io.File;
import java.io.PrintStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.seflowable.mock.PetalsSeFlowableRuntimeConfigurationMock;
import org.ow2.petals.cli.extension.seflowable.mock.ProcDefStruct;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;

/**
 * Unit test of {@link PurgeProcessInstance}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class PurgeProcessInstanceTest extends AbstractTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void usage() {
        final Command command = new PurgeProcessInstance();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws Exception {

        final Command command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>the process instance identifier is missing</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing process instance identifier(s).
     * </p>
     */
    @Test
    public void missingProcInst() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] {});

            fail("Exception 'CommandMissingOptionsException' is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, PurgeProcessInstance.PROC_INST_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>the process instance identifier value is missing,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process instance identifier.
     * </p>
     */
    @Test
    public void missingProcInstValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PurgeProcessInstance.PROC_INST_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>no process instance identifier is provided,</li>
     * <li>the input file for process instance identifiers is provided without value,</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process instance identifier.
     * </p>
     */
    @Test
    public void missingInputFileValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.INPUTFILE_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PurgeProcessInstance.INPUTFILE_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>a process instance identifier value is provided,</li>
     * <li>a input file for process instance identifiers is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the arguments in conflict.
     * </p>
     */
    @Test
    public void procInstArgumentsInConflict() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION, PROCESS_INSTANCE_11_1,
                    "-" + PurgeProcessInstance.INPUTFILE_SHORT_OPTION, "file.log" });

            fail("Exception 'CommandInvalidException' is not thrown");
        } catch (final CommandInvalidException e) {
            // Expected error
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>the component identifier value is missing</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process instance identifier.
     * </p>
     */
    @Test
    public void missingComponentIdValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION, PROCESS_INSTANCE_11_1,
                    "-" + AbstractCommand.COMPONENT_ID_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, AbstractCommand.COMPONENT_ID_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>flow instance ids of the process instance are correctly returned,</li>
     * <li>the process instance is deleted.</li>
     * </ul>
     */
    @Test
    public void purge() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION, PROCESS_INSTANCE_11_1 });

        // Check test results
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertTrue(output.contains(FLOW_INSTANCE_11_1_1));
        assertTrue(output.contains(FLOW_INSTANCE_11_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_11_2_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_21_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_21_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_21_2_1));
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>no process instance identifier is provided,</li>
     * <li>an input file containing existing process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>flow instance ids of the process instances are correctly returned,</li>
     * <li>the process instances is deleted.</li>
     * </ul>
     */
    @Test
    public void purgeFromInputFile() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());

        final File inputFile = this.tempFolder.newFile();
        try (final PrintStream ps = new PrintStream(inputFile);) {
            ps.println("# A comment");
            ps.println("// Another comment");
            ps.println(PROCESS_INSTANCE_11_1);
            ps.println();
            ps.println(PROCESS_INSTANCE_21_1);
        }

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(
                new String[] { "-" + PurgeProcessInstance.INPUTFILE_SHORT_OPTION, inputFile.getAbsolutePath() });

        // Check test results
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_2, "1")).size());

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertTrue(output.contains(FLOW_INSTANCE_11_1_1));
        assertTrue(output.contains(FLOW_INSTANCE_11_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_11_2_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_2));
        assertTrue(output.contains(FLOW_INSTANCE_21_1_1));
        assertTrue(output.contains(FLOW_INSTANCE_21_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_21_2_1));
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance", where:
     * </p>
     * <ul>
     * <li>no process instance identifier is provided,</li>
     * <li>an inexisting input file is provided as input file containing process instance identifiers,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the file not found.
     * </p>
     */
    @Test
    public void inputFileNotFound() throws Exception {

        // Set the environment
        final String inputFilename = "unexisting-input-file";
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.INPUTFILE_SHORT_OPTION, inputFilename });

            fail("Exception 'CommandInvalidArgumentException' is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, PurgeProcessInstance.INPUTFILE_SHORT_OPTION, inputFilename);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance", where:
     * </p>
     * <ul>
     * <li>no process instance identifier is provided,</li>
     * <li>a directory is provided as input file containing process instance identifiers,</li>
     * <li>no component id is provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the input file that is a directory.
     * </p>
     */
    @Test
    public void inputFileAsDirectory() throws Exception {

        // Set the environment
        final File inputFileAsDir = this.tempFolder.newFolder();

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.INPUTFILE_SHORT_OPTION,
                    inputFileAsDir.getAbsolutePath() });

            fail("Exception 'CommandInvalidArgumentException' is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, PurgeProcessInstance.INPUTFILE_SHORT_OPTION,
                    inputFileAsDir.getAbsolutePath());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output file is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>flow instance ids of the process instance are correctly returned into the provided file,</li>
     * <li>the process instance is deleted.</li>
     * </ul>
     */
    @Test
    public void purgeIntoFile() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());

        final File outputFile = this.tempFolder.newFile();

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION, PROCESS_INSTANCE_11_1,
                "-" + PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION, outputFile.getAbsolutePath() });

        // Check test results
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        final String outputFileContent = FileUtils.readFileToString(outputFile, Charset.defaultCharset());
        assertTrue(outputFileContent.contains(FLOW_INSTANCE_11_1_1));
        assertTrue(outputFileContent.contains(FLOW_INSTANCE_11_1_2));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_11_2_1));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_12_1_1));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_12_1_2));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_21_1_1));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_21_1_2));
        assertFalse(outputFileContent.contains(FLOW_INSTANCE_21_2_1));
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance", where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output file is provided without value.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing output file value.
     * </p>
     */
    @Test
    public void missingOutputFile() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PurgeProcessInstance.PROC_INST_SHORT_OPTION, PROCESS_INSTANCE_11_1,
                    "-" + PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance", where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>a file in inexisting directory is provided for the output file.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the output file not found.
     * </p>
     */
    @Test
    public void outputFileNotFound() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final File outputFileBaseDir = this.tempFolder.newFolder();
        final File outputFile = new File(new File(outputFileBaseDir, "inexisting-dir"), "file.out");

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION,
                    outputFile.getAbsolutePath() });

            fail("Exception 'CommandInvalidArgumentException' is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION,
                    outputFile.getAbsolutePath());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance", where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>no component id is provided,</li>
     * <li>an output directory is provided for the output file.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the output file that is a directory.
     * </p>
     */
    @Test
    public void outputFileAsDirectory() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final File outputFileAsDir = this.tempFolder.newFolder();

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new ListProcessInstances();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION,
                    outputFileAsDir.getAbsolutePath() });

            fail("Exception 'CommandInvalidArgumentException' is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, PurgeProcessInstance.OUTPUTFILE_SHORT_OPTION,
                    outputFileAsDir.getAbsolutePath());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" against argument management, where:
     * </p>
     * <ul>
     * <li>a valid and existing process instance identifier is provided,</li>
     * <li>no input file for process instance identifiers is provided,</li>
     * <li>an existing component id is correctly provided,</li>
     * <li>no output file is provided.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error occurs,</li>
     * <li>flow instance ids of the process instance are correctly returned,</li>
     * <li>the process instance is deleted.</li>
     * </ul>
     */
    @Test
    public void purgeWithComponentId() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        petalsSeFlowableRuntimeConfigurationMock.getFlowInstanceIds().putAll(FLOW_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMockOther = new PetalsSeFlowableRuntimeConfigurationMock();
        final String myComponentId = "my-comp-id";
        final String myProcDefKey = "my-proc-def";
        final ProcDefStruct myProcDef = new ProcDefStruct(myProcDefKey, "1");
        final String myProcInst = "my-proc-inst";
        final List<String> myProcInsts = new ArrayList<>();
        myProcInsts.add(myProcInst);
        petalsSeFlowableRuntimeConfigurationMockOther.getProcessInstanceIds().put(myProcDef, myProcInsts);
        final String myFlowInst = "my-flow-inst";
        final List<String> myFlowInsts = new ArrayList<>();
        myFlowInsts.add(myFlowInst);
        petalsSeFlowableRuntimeConfigurationMockOther.getFlowInstanceIds().put(myProcInst, myFlowInsts);
        this.jmxApi.addComponentClient(myComponentId, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMockOther, null);

        // Check test env
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "2")).size());
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_2, "1")).size());
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMockOther.getProcessInstanceIds().get(myProcDef).size());

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new PurgeProcessInstance();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_INST_SHORT_OPTION, myProcInst, "-" + COMPONENT_ID_SHORT_OPTION,
                myComponentId });

        // Check test results
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")).size());
        assertEquals(1, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "2")).size());
        assertEquals(2, petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_2, "1")).size());
        assertEquals(0, petalsSeFlowableRuntimeConfigurationMockOther.getProcessInstanceIds().get(myProcDef).size());

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertFalse("Empty output", output.isEmpty());

        assertTrue(output.contains(myFlowInst));
        assertFalse(output.contains(FLOW_INSTANCE_11_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_11_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_11_2_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_12_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_21_1_1));
        assertFalse(output.contains(FLOW_INSTANCE_21_1_2));
        assertFalse(output.contains(FLOW_INSTANCE_21_2_1));
    }

    /**
     * <p>
     * Test of the command "se-flowable.purge-process-instance" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void notConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new PurgeProcessInstance();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
