/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingArgumentException;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;
import static org.ow2.petals.cli.extension.seflowable.AbstractCommand.COMPONENT_ID_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_KEY_SHORT_OPTION;
import static org.ow2.petals.cli.extension.seflowable.ListProcessInstances.PROC_DEF_VER_SHORT_OPTION;

import java.util.Collections;

import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.extension.seflowable.mock.PetalsSeFlowableRuntimeConfigurationMock;
import org.ow2.petals.cli.extension.seflowable.mock.ProcDefStruct;
import org.ow2.petals.cli.shell.command.Connect;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;
import org.ow2.petals.se.flowable.clientserver.api.admin.exception.ProcessDefinitionNotFoundException;
import org.ow2.petals.se.flowable.clientserver.api.admin.exception.ProcessInstanceExistForDefinitionException;

/**
 * Unit test of {@link UndeployProcessDefinition}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class UndeployProcessDefinitionTest extends AbstractTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void usage() {
        final Command command = new UndeployProcessDefinition();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws Exception {

        final Command command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition key is missing,</li>
     * <li>the process definition version is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing process definition key and version.
     * </p>
     */
    @Test
    public void missingProcDef() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

            fail("Exception 'CommandMissingOptionsException' is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, PROC_DEF_KEY_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition key is missing,</li>
     * <li>an existing process definition version is correctly provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing process definition key.
     * </p>
     */
    @Test
    public void missingProcDefKey() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

            fail("Exception 'CommandMissingOptionsException' is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, PROC_DEF_KEY_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition key value is missing,</li>
     * <li>an existing process definition version is correctly provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process definition key.
     * </p>
     */
    @Test
    public void missingProcDefKeyValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PROC_DEF_KEY_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>an existing process definition key correctly provided,</li>
     * <li>the process definition version is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing process definition version.
     * </p>
     */
    @Test
    public void missingProcDefVer() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1 });

            fail("Exception 'CommandMissingOptionsException' is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, PROC_DEF_VER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>an existing process definition key correctly provided,</li>
     * <li>the process definition version value is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process definition version.
     * </p>
     */
    @Test
    public void missingProcDefVerValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, PROC_DEF_VER_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>the process definition version is unparseable</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid value.
     * </p>
     */
    @Test
    public void unparsableProcDefVer() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        final String unparsableValue = "123aze456";
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "procDefKey",
                    "-" + PROC_DEF_VER_SHORT_OPTION, unparsableValue });

            fail("Exception 'CommandUnparsableArgumentException' is not thrown");
        } catch (final CommandUnparsableArgumentException e) {
            assertEquals(PROC_DEF_VER_SHORT_OPTION, e.getOption().getOpt());
            assertEquals(unparsableValue, e.getUnparsableValue());
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" against argument management, where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>the component identifier value is missing</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing value for process instance identifier.
     * </p>
     */
    @Test
    public void missingComponentIdValue() throws Exception {

        // Set the environment
        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1", "-" + AbstractCommand.COMPONENT_ID_SHORT_OPTION });

            fail("Exception 'CommandMissingArgumentException' is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertCommandMissingArgumentException(e, AbstractCommand.COMPONENT_ID_SHORT_OPTION);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>no process instance exists.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the process definition is undeployed.
     * </p>
     */
    @Test
    public void undeployProcDef() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .put(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1"), Collections.EMPTY_LIST);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                "-" + PROC_DEF_VER_SHORT_OPTION, "1" });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", output.isEmpty());
        assertNull("Process definition not undeployed", petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds()
                .get(new ProcDefStruct(PROCESS_DEFINITION_KEY_1, "1")));
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>an existing component id is correctly provided,</li>
     * <li>no process instance exists.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the process definition is undeployed.
     * </p>
     */
    @Test
    public void undeployProcDefWithComponentId() throws Exception {

        // Set the environment
        final String myProcDefKey = "myProcDefKey";
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMockDefault = new PetalsSeFlowableRuntimeConfigurationMock();
        // Process instances initialized on SE Flowable not requested
        petalsSeFlowableRuntimeConfigurationMockDefault.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMockDefault, null);
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMockOther = new PetalsSeFlowableRuntimeConfigurationMock();
        final String myComponentId = "my-comp-id";
        // No process instances initialized on SE Flowable requested
        petalsSeFlowableRuntimeConfigurationMockOther.getProcessInstanceIds().put(new ProcDefStruct(myProcDefKey, "1"),
                Collections.EMPTY_LIST);
        this.jmxApi.addComponentClient(myComponentId, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMockOther, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());
        command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, myProcDefKey, "-" + PROC_DEF_VER_SHORT_OPTION,
                "1", "-" + COMPONENT_ID_SHORT_OPTION, myComponentId });

        // Check test result
        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", output.isEmpty());
        assertNull("Process definition not undeployed", petalsSeFlowableRuntimeConfigurationMockOther
                .getProcessInstanceIds().get(new ProcDefStruct(myProcDefKey, "1")));
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition", where:
     * </p>
     * <ul>
     * <li>an existing process definition key is correctly provided,</li>
     * <li>an existing process definition version is correctly provided,</li>
     * <li>no component id is provided,</li>
     * <li>process instances exist.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs because of existing process instances
     * </p>
     */
    @Test
    public void undeployProcDefWithProcInsts() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        petalsSeFlowableRuntimeConfigurationMock.getProcessInstanceIds().putAll(PROCESS_INSTANCES);
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());

        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, PROCESS_DEFINITION_KEY_1,
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1" });
        } catch (final CommandException e) {
            // Check test result
            assertNotNull(e.getCause());
            assertTrue(e.getCause() instanceof ProcessInstanceExistForDefinitionException);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition", where:
     * </p>
     * <ul>
     * <li>an unexisting process definition key is correctly provided,</li>
     * <li>an unexisting process definition version is correctly provided,</li>
     * <li>no component id is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs because of unexisting process definition
     * </p>
     */
    @Test
    public void undeployUnexistingProcDef() throws Exception {

        // Set the environment
        final PetalsSeFlowableRuntimeConfigurationMock petalsSeFlowableRuntimeConfigurationMock = new PetalsSeFlowableRuntimeConfigurationMock();
        this.jmxApi.addComponentClient(AbstractCommand.DEFAULT_COMPONENT_ID, ComponentType.ENGINE, null, null,
                petalsSeFlowableRuntimeConfigurationMock, null);

        final Connect connect = new Connect();
        connect.setShell(this.dummyShellWrapper.getShell());
        connect.connect(new AuthenticatedConnectionParametersImpl("localhost", 7700, "petals", "petals", null), false);

        // Execute test
        final AbstractCommand command = new UndeployProcessDefinition();
        command.setShell(this.dummyShellWrapper.getShell());
        // inject JMX client because petals admin does not provide the abstractions we need
        command.setJMXClient(this.jmxApi.connect());

        try {
            command.execute(new String[] { "-" + PROC_DEF_KEY_SHORT_OPTION, "unexisting-proc-def",
                    "-" + PROC_DEF_VER_SHORT_OPTION, "1" });
        } catch (final CommandException e) {
            // Check test result
            assertNotNull(e.getCause());
            assertTrue(e.getCause() instanceof ProcessDefinitionNotFoundException);
        }
    }

    /**
     * <p>
     * Test of the command "se-flowable.undeploy-process-definition" when no connection is established to run the
     * command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void notConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new UndeployProcessDefinition();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
