/**
 * Copyright (c) 2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.seflowable.mock;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.se.flowable.clientserver.api.admin.AdminRuntimeService;
import org.ow2.petals.se.flowable.clientserver.api.admin.exception.ProcessDefinitionNotFoundException;
import org.ow2.petals.se.flowable.clientserver.api.admin.exception.ProcessInstanceExistForDefinitionException;
import org.ow2.petals.se.flowable.clientserver.api.admin.exception.ProcessInstanceNotFoundException;

/**
 * A mock of {@link AdminRuntimeService} for unit testing
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class PetalsSeFlowableRuntimeConfigurationMock implements AdminRuntimeService {

    private final Map<ProcDefStruct, List<String>> procInstIds = new ConcurrentHashMap<>();

    private final Map<String, List<String>> flowInstIds = new ConcurrentHashMap<>();

    @Override
    public List<String> listPurgeableProcessInstances(final String processDefinitionKey,
            final int processDefinitionVersion) throws PetalsException {

        if (processDefinitionKey == null || processDefinitionKey.trim().isEmpty()) {
            final List<String> result = new ArrayList<>();
            for (final List<String> procInsts : this.procInstIds.values()) {
                result.addAll(procInsts);
            }
            return result;
        } else {
            final List<String> result = this.procInstIds
                    .get(new ProcDefStruct(processDefinitionKey, String.valueOf(processDefinitionVersion)));
            if (result == null) {
                return Collections.EMPTY_LIST;
            } else {
                return result;
            }
        }
    }

    @Override
    public List<String> purgeProcessInstance(final String procInstId, final boolean returnsCorrelatedFlows)
            throws PetalsException {

        // Delete the process instance
        for (final List<String> procInsts : this.procInstIds.values()) {
            if (procInsts.remove(procInstId)) {
                break;
            }
        }

        // Returns associated flow instances
        final List<String> flowInstIds = this.flowInstIds.remove(procInstId);
        if (flowInstIds == null) {
            throw new ProcessInstanceNotFoundException(procInstId);
        } else {
            if (returnsCorrelatedFlows) {
                return flowInstIds;
            } else {
                return Arrays.asList(flowInstIds.get(0));
            }
        }
    }

    @Override
    public void undeployProcessDefinition(final String processDefinitionKey, final int processDefinitionVersion)
            throws PetalsException {
        assert processDefinitionKey != null;
        assert !processDefinitionKey.trim().isEmpty();

        final ProcDefStruct procDef = new ProcDefStruct(processDefinitionKey, String.valueOf(processDefinitionVersion));
        final List<String> result = this.procInstIds
                .get(procDef);
        if (result != null && result.size() > 0) {
            throw new ProcessInstanceExistForDefinitionException(processDefinitionKey, processDefinitionVersion);
        } else if (result != null && result.size() == 0) {
            this.procInstIds.remove(procDef);
        } else {
            throw new ProcessDefinitionNotFoundException(processDefinitionKey, processDefinitionVersion);
        }

    }

    public Map<ProcDefStruct, List<String>> getProcessInstanceIds() {
        return this.procInstIds;
    }

    public Map<String, List<String>> getFlowInstanceIds() {
        return this.flowInstIds;
    }
}
